dnl $Header: /u/spert/src/fltvec/RCS/aclocal.m4,v 1.9 1996/04/11 20:25:21 davidj Exp $
dnl
dnl Autoconf macros for SPERT libraries and programs

dnl ****************************************************************
dnl A check for whether we are using gcc
dnl ****************************************************************

AC_DEFUN(ACSPERT_C_ISGCC,
[AC_MSG_CHECKING(whether we are using GNU C)
AC_CACHE_VAL(ac_cv_prog_gcc,
[dnl The semicolon is to pacify NeXT's syntax-checking cpp.
cat > conftest.c <<EOF
#ifdef __GNUC__
  yes;
#endif
EOF
if ${CC-cc} -E conftest.c 2>&AC_FD_CC | egrep yes >/dev/null 2>&1; then
  ac_cv_prog_gcc=yes
else
  ac_cv_prog_gcc=no
fi])dnl
AC_MSG_RESULT($ac_cv_prog_gcc)
if test $ac_cv_prog_gcc = yes; then
  GCC=yes
fi
])

dnl ****************************************************************
dnl A search for the best floating point compiler on the system, along with
dnl appropriate flags to use during compilation
dnl ****************************************************************

AC_DEFUN(ACSPERT_PROG_CC_FASTFP,
[AC_BEFORE([$0], [AC_PROG_CPP])dnl
AC_CHECK_PROGS(CC, acc xlc c89, cc)
AC_MSG_CHECKING(for a fast C compiler)
AC_MSG_RESULT($CC)
ACSPERT_C_ISGCC
dnl work out the best compiler flags
if test "${CFLAGS+set}" != "set"
then
  if test "${GCC}" = "yes"
  then
    CFLAGS="-O2 -g -Wall"
  else
    if test $CC = "acc"
    then
      CFLAGS="-O2 -cg92 -fsingle -dalign -g +w"
    else
      CFLAGS="-O2 -g"
    fi
  fi
fi
])

dnl ****************************************************************
dnl A combination of AC_CHECK_TOOL and AC_PROG_CC that checks for cross
dnl C compilers
dnl ****************************************************************

AC_DEFUN(ACSPERT_TOOL_CC,
[AC_BEFORE([$0], [AC_PROG_CPP])dnl
AC_CHECK_TOOL(CC, gcc, cc)
ACSPERT_C_ISGCC
dnl Work out a suitable value for CFLAGS
if test "${GCC}" = yes; then
  if test "${CFLAGS+set}" != set; then
    AC_MSG_CHECKING(whether ${CC-cc} accepts -g)
AC_CACHE_VAL(ac_cv_prog_gcc_g,
[echo 'void f(){}' > conftest.c
if test -z "`${CC-cc} -g -c conftest.c 2>&1`"; then
  ac_cv_prog_gcc_g=yes
else
  ac_cv_prog_gcc_g=no
fi
rm -f conftest*
])dnl
    AC_MSG_RESULT($ac_cv_prog_gcc_g)
    if test $ac_cv_prog_gcc_g = yes; then
      CFLAGS="-g -O -Wall"
    else
      CFLAGS="-O -Wall"
    fi
  fi
else
  test "${CFLAGS+set}" = set || CFLAGS="-g"
fi
])

dnl ****************************************************************
dnl A multiple name version of AC_CHECK_TOOL
dnl ****************************************************************

dnl ACSPERT_CHECK_TOOLS(VARIABLE, TOOLS-TO-CHECK-FOR [, VALUE-IF-NOT-FOUND])
AC_DEFUN(ACSPERT_CHECK_TOOLS,
[AC_REQUIRE([AC_CHECK_TOOL_PREFIX])dnl
for ac_prog in $2
do
  AC_CHECK_PROG($1, ${ac_tool_prefix}[$]ac_prog, ${ac_tool_prefix}[$]ac_prog, )
  test -n "[$]$1" && break
done
if test -z "[$]$1"
then
  for ac_prog in $2
do
AC_CHECK_PROG($1, [$]ac_prog, [$]ac_prog, )
test -n "[$]$1" && break
done
fi
ifelse([$3], , , [test -n "[$]$1" || $1="$3"
])])

dnl ****************************************************************
dnl A combination of AC_CHECK_TOOLS and AC_PROG_CXX that checks for cross
dnl C++ compilers
dnl ****************************************************************

AC_DEFUN(ACSPERT_TOOL_CXX,
[AC_BEFORE([$0], [AC_PROG_CXXCPP])dnl
ACSPERT_CHECK_TOOLS(CXX, $CCC c++ g++ gcc CC cxx, gcc)
AC_MSG_CHECKING(whether we are using GNU C++)
AC_CACHE_VAL(ac_cv_prog_gxx,
[dnl The semicolon is to pacify NeXT's syntax-checking cpp.
cat > conftest.C <<EOF
#ifdef __GNUC__
  yes;
#endif
EOF
if ${CXX-g++} -E conftest.C 2>&AC_FD_CC | egrep yes >/dev/null 2>&1; then
  ac_cv_prog_gxx=yes
else
  ac_cv_prog_gxx=no
fi])dnl
AC_MSG_RESULT($ac_cv_prog_gxx)
if test $ac_cv_prog_gxx = yes; then
  GXX=yes
  if test "${CXXFLAGS+set}" != set; then
    AC_MSG_CHECKING(whether ${CXX-g++} accepts -g)
AC_CACHE_VAL(ac_cv_prog_gxx_g,
[echo 'void f(){}' > conftest.cc
if test -z "`${CXX-g++} -g -c conftest.cc 2>&1`"; then
  ac_cv_prog_gxx_g=yes
else
  ac_cv_prog_gxx_g=no
fi
rm -f conftest*
])dnl
    AC_MSG_RESULT($ac_cv_prog_gxx_g)
    if test $ac_cv_prog_gxx_g = yes; then
      CXXFLAGS="-g -O -Wall"
    else
      CXXFLAGS="-O -Wall"
    fi
  fi
else
  GXX=
  test "${CXXFLAGS+set}" = set || CXXFLAGS="-g"
fi
])

dnl ****************************************************************
dnl Work out how to run programs - empty command on a workstation,
dnl but an appropriate spertrun command for SPERT
dnl ****************************************************************
AC_DEFUN(ACSPERT_PROG_RUN,
[
RUN=
AC_MSG_CHECKING(how to run test programs)
if test "$host_cpu" = "torrent0"; then
  RUN=spertrun
fi
AC_SUBST(RUN)
AC_MSG_RESULT(${RUN:-host})
])

dnl ****************************************************************
dnl Work out how to assembler programs that need preprocessing.
dnl ****************************************************************
AC_DEFUN(ACSPERT_TOOL_AS,
[
AC_MSG_CHECKING(how to preprocess + assemble source)
dnl AC_REQUIRE(ACSPERT_TOOL_CC)
if test "${AS+set}" != set; then
  if test "$ac_cv_prog_gcc" = yes; then
    AS="${CC} -c"
  else
    AS=gcc -c
  fi
fi
AC_MSG_RESULT($AS)
AC_SUBST(AS)
])


