/*
    $Header: /u/spert/src/fltvec/testsuite/RCS/convol_vfvf_vf_test.c,v 1.3 1995/07/05 23:46:30 davidj Exp $

    Test convolution routines
*/

#include <stddef.h>
#include <stdlib.h>

#include "fltvec_test.h"

#ifdef DOUBLE

#define FP_TYPE double
#define TEST_FUNC convol_vdvd_vd
#define NAIVE_TEST_FUNC nv_convol_vdvd_vd
#define RTST_PADVEC_CLONE_V rtst_padvec_clone_vd
#define RTST_PADVEC_NEW_V rtst_padvec_new_vd
#define RTST_PADVEC_DEL_V rtst_padvec_del_vd
#define RTST_URANDI_I32I32_V rtst_urandi_i32i32_vd
#define RTST_CHECKEQ_VV rtst_checkeq_vdvd
#define MALLOC_V malloc_vd
#define FREE_V free_vd

extern void nv_convol_vdvd_vd(size_t, size_t, const double *,
                              const double *, double *);

extern void convol_vdvd_vd(size_t, size_t, const double *,
			   const double *, double *);
#else

#define FP_TYPE float
#define TEST_FUNC convol_vfvf_vf
#define NAIVE_TEST_FUNC nv_convol_vfvf_vf
#define RTST_PADVEC_CLONE_V rtst_padvec_clone_vf
#define RTST_PADVEC_NEW_V rtst_padvec_new_vf
#define RTST_PADVEC_DEL_V rtst_padvec_del_vf
#define RTST_URANDI_I32I32_V rtst_urandi_i32i32_vf
#define RTST_CHECKEQ_VV rtst_checkeq_vfvf
#define MALLOC_V malloc_vf
#define FREE_V free_vf

extern void nv_convol_vfvf_vf(size_t, size_t, const float *,
                              const float *, float *);

extern void convol_vfvf_vf(size_t, size_t, const float *,
			   const float *, float *);
#endif

/* perform one test given inputs and expected outputs */

void
one_test(size_t h_len, size_t x_len, const FP_TYPE *h, const FP_TYPE *x,
         const FP_TYPE *y) {
  /* create padded copies of inputs */
  FP_TYPE *c_h = RTST_PADVEC_CLONE_V(h_len, h);
  FP_TYPE *c_x = RTST_PADVEC_CLONE_V(x_len, x);

  /* create padded output vector */
  FP_TYPE *c_y = RTST_PADVEC_NEW_V(x_len);

  TEST_FUNC(h_len, x_len, c_h, c_x, c_y);

  /* check result */
  RTST_CHECKEQ_VV(x_len, y, c_y);

  /* check inputs not corruputed */
  RTST_CHECKEQ_VV(x_len, x, c_x);
  RTST_CHECKEQ_VV(h_len, h, c_h);

  /* free copies, and check padding */
  RTST_PADVEC_DEL_V(c_x);
  RTST_PADVEC_DEL_V(c_y);
  RTST_PADVEC_DEL_V(c_h);
}

void
test_1x1() {
#ifdef DOUBLE
  static FP_TYPE h[1] = { 2.0 };
  static FP_TYPE x[1] = { -4.0 };
  static FP_TYPE y[1] = { -8.0 };
#else
  static FP_TYPE h[1] = { 2.0f };
  static FP_TYPE x[1] = { -4.0f };
  static FP_TYPE y[1] = { -8.0f };
#endif
  rtst_start("test_1x1");
  one_test(1, 1, h, x, y);
  rtst_passed();
}

void
test_4x9() {
#ifdef DOUBLE
  static FP_TYPE h[4] = { 3.0, 1.0, -1.0, -3.0 };
  static FP_TYPE x[9] = { -4.0, -3.0, -2.0, -1.0, 0.0,
                        1.0, 2.0, 3.0, 4.0 };
  static FP_TYPE y[9] = { -12.0, -13.0, -5.0, 10.0, 10.0, 10.0,
                        10.0, 10.0, 10.0 };
#else
  static FP_TYPE h[4] = { 3.0f, 1.0f, -1.0f, -3.0f };
  static FP_TYPE x[9] = { -4.0f, -3.0f, -2.0f, -1.0f, 0.0f,
                        1.0f, 2.0f, 3.0f, 4.0f };
  static FP_TYPE y[9] = { -12.0f, -13.0f, -5.0f, 10.0f, 10.0f, 10.0f,
                        10.0f, 10.0f, 10.0f };
#endif
  rtst_start("test_4x9");
  one_test(4, 9, h, x, y);
  rtst_passed();
}

void
run_directed_tests(void) {
  test_1x1();
  test_4x9();
}

void
run_random_tests(void) {
  const int numtests = rtst_numtests;
  const size_t maxfiltlen = rtst_sizetests;
  const size_t maxsiglen = 2 * rtst_sizetests;
  FP_TYPE *h = MALLOC_V(maxfiltlen);
  FP_TYPE *x = MALLOC_V(maxsiglen);
  FP_TYPE *y = MALLOC_V(maxsiglen);

  int i;
  for ( i = 0; i < numtests; i++ ) {
    size_t filt = (size_t) rtst_urand_i32i32_i32(1, maxfiltlen);
    size_t sig;
    
    do {
      sig  = (size_t) rtst_urand_i32i32_i32(1, maxsiglen);
    } while (sig < filt);

    RTST_URANDI_I32I32_V(sig,  -7, +7, x);
    RTST_URANDI_I32I32_V(filt, -7, +7, h);

    NAIVE_TEST_FUNC(filt, sig, h, x, y);

    rtst_log("random filt=%i   sig=%i\n", filt, sig);
    one_test(filt, sig, h, x, y);
  }

  FREE_V(h);
  FREE_V(x);
  FREE_V(y);
}

int
main(int argc, char **argv) {
  rtst_args(argc, argv);

  rtst_start("Directed tests on convol");
  run_directed_tests();
  rtst_passed();

  rtst_start("Random tests against naive loop");
  run_random_tests();
  rtst_passed();

  rtst_exit();
  return 1;
}

