/*
   $Header: /u/spert/src/fltvec/testsuite/RCS/maxmin_vf_test.c,v 1.3 1996/04/01 23:36:37 davidj Exp $

   Test floating point vector sum
*/

#include <stddef.h>
#include <stdlib.h>

#include "fltvec_test.h"


typedef size_t (*rtn_t)(size_t len, const float* in);

/* Performs one test given input values and expected output. */

void
onetest(rtn_t rtn, size_t len,
	const float* in, size_t correct)
{
    /* Create padded copies of inputs. */
    float* c_in = rtst_padvec_clone_vf(len, in);
    size_t res;

    /* Run routine. */
    res = rtn(len, c_in);

    /* Check result. */
    if (res!=correct)
	rtst_failed("%s:%d: wrong, correct=%lu actual=%lu.\n",
		    __FILE__, __LINE__, correct, res);

    /* Check input not corrupted. */
    rtst_checkeq_vfvf(len, in, c_in);

    /* Free and check padding. */
    rtst_padvec_del_vf(c_in);
}

enum
{
    NAN = 0x7f800001,
    NNAN = 0xffffffff
};

void
random_tests(rtn_t nv_rtn, rtn_t test_rtn)
{
    const int numtests = rtst_numtests;
    const size_t maxlen = rtst_sizetests;
    float* in = malloc_vf(maxlen);
    size_t res;
    int i;

    for (i=0; i<numtests; i++)
    {
        size_t len = (size_t) rtst_urand_i32i32_i32(1, maxlen);
        rtst_urand_ff_vf(len, -1e2, 1e2, in);
        res = nv_rtn(len, in);
        rtst_log("len=%d\n", len);
        onetest(test_rtn, len, in, res);
    }
    free_vf(in);
}


void
imax_directed_tests(rtn_t rtn)
{
    const float in1[] = { 42.0};
    const float in2[] = { 1.0, 1.1 };
    const float in3[] = { 1.1, 1.0 };
    const fltv_uint32_t in4[] = { NAN };
    const fltv_uint32_t in5[] = { 0, (fltv_uint32_t) NNAN };
    const fltv_uint32_t in6[33] = {	/* NaN in element 0 */
	NAN, 0, 0, 0, 0, 0, 0, 0,0, 0, 0, 0,0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,0, 0, 0, 0,0, 0, 0, 0,
	0
    };
    const fltv_uint32_t in7[33] = {	/* NaN in element 32 */
	0, 0, 0, 0, 0, 0, 0, 0,0, 0, 0, 0,0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,0, 0, 0, 0,0, 0, 0, 0,
	NAN
    };
    const fltv_uint32_t in8[65] = {	/* NaN in element 32 */
	0, 0, 0, 0, 0, 0, 0, 0,0, 0, 0, 0,0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,0, 0, 0, 0,0, 0, 0, 0,
	NAN, 0, 0, 0, 0, 0, 0, 0,0, 0, 0, 0,0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,0, 0, 0, 0,0, 0, 0, 0,
	0
    };
    const fltv_uint32_t in9[65] = {	/* NaN in element 64 */
	0, 0, 0, 0, 0, 0, 0, 0,0, 0, 0, 0,0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,0, 0, 0, 0,0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,0, 0, 0, 0,0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,0, 0, 0, 0,0, 0, 0, 0,
	NAN
    };
    const float in10[4] = { 1.0, 2.0, -0.5, -3.0 };

    rtst_log("Test1.\n");
    onetest(rtn, 1, in1, 0);
    rtst_log("Test2.\n");
    onetest(rtn, 2, in2, 1);
    rtst_log("Test3.\n");
    onetest(rtn, 2, in3, 0);
    rtst_log("Test4.\n");
    onetest(rtn, 1, (float *)in4, FLTV_SIZET_BAD);
    rtst_log("Test5.\n");
    onetest(rtn, 2, (float *)in5, FLTV_SIZET_BAD);
    rtst_log("Test6.\n");
    onetest(rtn, 33, (float *)in6, FLTV_SIZET_BAD);
    rtst_log("Test7.\n");
    onetest(rtn, 33, (float *)in7, FLTV_SIZET_BAD);
    rtst_log("Test8.\n");
    onetest(rtn, 65, (float *)in8, FLTV_SIZET_BAD);
    rtst_log("Test9.\n");
    onetest(rtn, 65, (float *)in9, FLTV_SIZET_BAD);
    rtst_log("Test10.\n");
    onetest(rtn, 4, in10, 1);
}

void
imin_directed_tests(rtn_t rtn)
{
    const float in1[] = { 42.0};
    const float in2[] = { 1.0, 1.1 };
    const float in3[] = { 1.1, 1.0 };
    const float in4[] = { 1.0, 4.0, -0.5, -3.0 };

    rtst_log("Test1.\n");
    onetest(rtn, 1, in1, 0);
    rtst_log("Test2.\n");
    onetest(rtn, 2, in2, 0);
    rtst_log("Test3.\n");
    onetest(rtn, 2, in3, 1);
    rtst_log("Test4.\n");
    onetest(rtn, 4, in4, 3);
}

void
iabsmax_directed_tests(rtn_t rtn)
{
    const float in1[] = { 42.0};
    const float in2[] = { 1.0, 1.1 };
    const float in3[] = { 1.1, 1.0 };
    const float in4[] = { 1.0, 4.0, -0.5, -5.0 };

    rtst_log("Test1.\n");
    onetest(rtn, 1, in1, 0);
    rtst_log("Test2.\n");
    onetest(rtn, 2, in2, 1);
    rtst_log("Test3.\n");
    onetest(rtn, 2, in3, 0);
    rtst_log("Test4.\n");
    onetest(rtn, 4, in4, 3);
}

void
iabsmin_directed_tests(rtn_t rtn)
{
    const float in1[] = { 42.0};
    const float in2[] = { 1.0, 1.1 };
    const float in3[] = { 1.1, 1.0 };
    const float in4[] = { 1.0, 4.0, -0.5, -5.0 };

    rtst_log("Test1.\n");
    onetest(rtn, 1, in1, 0);
    rtst_log("Test2.\n");
    onetest(rtn, 2, in2, 0);
    rtst_log("Test3.\n");
    onetest(rtn, 2, in3, 1);
    rtst_log("Test4.\n");
    onetest(rtn, 4, in4, 2);
}

void
imax_tests(void)
{
    rtst_start("imax_vf_u");
    rtst_log("Testing nv.\n");
    imax_directed_tests(nv_imax_vf_u);

#ifdef CHECK_TOR
    rtst_log("Testing tor.\n");
    imax_directed_tests(tor_imax_vf_u);
    rtst_log("Random testing tor.\n");
    random_tests(nv_imax_vf_u, tor_imax_vf_u);
#endif

    rtst_passed();
}

void
imin_tests(void)
{
    rtst_start("imin_vf_u");
    rtst_log("Testing nv.\n");
    imin_directed_tests(nv_imin_vf_u);

#ifdef CHECK_TOR
    rtst_log("Testing tor.\n");
    imin_directed_tests(tor_imin_vf_u);
    rtst_log("Random testing tor.\n");
    random_tests(nv_imin_vf_u, tor_imin_vf_u);
#endif

    rtst_passed();
}

void
iabsmax_tests(void)
{
    rtst_start("iabsmax_vf_u");
    rtst_log("Testing nv.\n");
    iabsmax_directed_tests(nv_iabsmax_vf_u);

#ifdef CHECK_TOR
    rtst_log("Testing tor.\n");
    iabsmax_directed_tests(tor_iabsmax_vf_u);
    rtst_log("Random testing tor.\n");
    random_tests(nv_iabsmax_vf_u, tor_iabsmax_vf_u);
#endif

    rtst_passed();
}

void
iabsmin_tests(void)
{
    rtst_start("iabsmin_vf_u");
    rtst_log("Testing nv.\n");
    iabsmin_directed_tests(nv_iabsmin_vf_u);

#ifdef CHECK_TOR
    rtst_log("Testing tor.\n");
    iabsmin_directed_tests(tor_iabsmin_vf_u);
    rtst_log("Random testing tor.\n");
    random_tests(nv_iabsmin_vf_u, tor_iabsmin_vf_u);
#endif

    rtst_passed();
}


int
main(int argc, char *argv[])
{
    rtst_args(argc, argv);
    imax_tests();
    imin_tests();
    iabsmax_tests();
    iabsmin_tests();
    rtst_exit();

    return 0;
}



