const char *muladd_vffvf_vf_test_rcsid = "$Header: /u/spert/src/fltvec/testsuite/RCS/muladd_vffvf_vf_test.c,v 1.2 1995/07/05 23:47:14 davidj Exp $";
/*
   Test muladd_vffvf_vf routine.
*/

#include <stddef.h>
#include <stdlib.h>

#include "fltvec_test.h"

/* Performs one test given input values and expected output. */
/* Tests case where resvec != xvec && resvec != yvec. */

typedef void (*rtn_t)(size_t len, const float* x, float a, const float* y,
		      float* res);

void
one_test(rtn_t rtn, size_t n,
         const float* xvec, const float a, const float *yvec,
         const float* resvec)
{
    /* Create padded copies of inputs. */
    float* c_xvec = rtst_padvec_clone_vf(n, xvec);
    float* c_yvec = rtst_padvec_clone_vf(n, yvec);
    float* c_resvec = rtst_padvec_new_vf(n);

    /* Run routine. */
    rtn(n, c_xvec, a, c_yvec, c_resvec);

    /* Check result. */
    rtst_checkeq_vfvf(n, c_resvec, resvec);

    /* Check input not corrupted. */
    rtst_checkeq_vfvf(n, c_xvec, xvec);
    rtst_checkeq_vfvf(n, c_yvec, yvec);

    /* Free and check padding. */
    rtst_padvec_del_vf(c_xvec);
    rtst_padvec_del_vf(c_yvec);
    rtst_padvec_del_vf(c_resvec);
}

void
test_1(rtn_t rtn)
{
    static const float x[] = {  5.0 };
    static const float y[] = { 12.0 };
    static const float r[] = { 22.0 };

    rtst_log("Running test_1().\n");
    one_test(rtn, 1, x, 2.0, y, r);
    rtst_log("Passed test_1().\n");
}

void
test_3(rtn_t rtn)
{
    static const float x[] = { 7.0, -5.0, +4.0 };
    static const float y[] = { 1.0,  3.0, -2.0 };
    static const float r[] = { 29.0, -17.0, 14.0 };

    rtst_log("Running test_3().\n");
    one_test(rtn, 3, x, 4.0, y, r);
    rtst_log("Passed test_3().\n");
}

void
run_directed_tests(rtn_t rtn)
{
    rtst_log("\nRunning directed tests.\n");
    test_1(rtn);
    test_3(rtn);
    rtst_log("Passed directed tests.\n\n");
}

void
run_random_tests(rtn_t rtn)
{
    const int numtests = rtst_numtests;
    const size_t maxlen = rtst_sizetests;
    float *xvec = malloc_vf(maxlen);
    float *yvec = malloc_vf(maxlen);
    float *resvec = malloc_vf(maxlen);
    int i;

    rtst_log("\nStarting random tests against naive code.\n");

    for (i=0; i<numtests; i++)
    {
        const size_t n = (size_t) rtst_urand_i32i32_i32(0, maxlen); /* FIXME */
        const float s = (float) rtst_urand_i32i32_i32(-7, +7);

        rtst_urandi_i32i32_vf(n, -7, +7, xvec);
        rtst_urandi_i32i32_vf(n, -7, +7, yvec);

        nv_muladd_vffvf_vf(n, xvec, s, yvec, resvec);

        rtst_log("n=%ld\n", n);

        one_test(rtn, n, xvec, s, yvec, resvec);
    }

    rtst_log("Passed random tests against naive code.\n\n");

    free_vf(xvec);
    free_vf(yvec);
    free_vf(resvec);
}

int
main(int argc, char *argv[])
{
    rtst_args(argc, argv);

    rtst_start("nv_muladd_vffvf_vf");
    run_directed_tests(nv_muladd_vffvf_vf);
    rtst_passed();

#ifdef CHECK_PHP
    rtst_start("php_muladd_vffvf_vf");
    run_directed_tests(php_muladd_vffvf_vf);
    run_random_tests(php_muladd_vffvf_vf);
    rtst_passed();
#endif


#ifdef CHECK_TOR
    rtst_start("tor_muladd_vffvf_vf");
    run_directed_tests(tor_muladd_vffvf_vf);
    run_random_tests(tor_muladd_vffvf_vf);
    rtst_passed();
#endif
    
    rtst_exit();

    return 0;
}
