/*
   $Header: /u/spert/src/fltvec/testsuite/RCS/mulsum_vfvf_f_test.c,v 1.3 1996/03/22 03:38:18 davidj Exp $

   Test mulsum_vfvf_f routine.
*/

#include <stddef.h>
#include <stdlib.h>

#include "fltvec_test.h"

typedef float (*rtn_t)(size_t len, const float* a, const float* b);


/* Performs one test given input values and expected output. */

void
one_test(rtn_t rtn, size_t n, const float*veca, const float*vecb,
	 const float res)
{
    /* Create padded copies of inputs. */
    float* c_veca = rtst_padvec_clone_vf(n, veca);
    float* c_vecb = rtst_padvec_clone_vf(n, vecb);

    float c_res = rtn(n, c_veca, c_vecb);

    /* Check result. */
    rtst_assert(c_res == res);

    /* Check input not corrupted. */
    rtst_checkeq_vfvf(n, c_veca, veca);
    rtst_checkeq_vfvf(n, c_vecb, vecb);

    /* Free and check padding. */
    rtst_padvec_del_vf(c_veca);
    rtst_padvec_del_vf(c_vecb);
}

void
length_1_test(rtn_t rtn)
{
    static float a[1] = { -1.0 };
    static float b[1] = { 3.0 };
    rtst_log("started length_1_test\n");
    one_test(rtn, 1, a, b, -3.0f);
    rtst_log("finished length_1_test\n");
}

void
length_2_test(rtn_t rtn)
{
    static float a[2] = { 4.0, -3.0 };
    static float b[2] = { 3.0, -3.0 };
    rtst_log("started length_2_test\n");
    one_test(rtn, 2, a, b, 21.0f);
    rtst_log("finished length_2_test\n");
}

void
length_9_test(rtn_t rtn)
{
    static float a[9] = { 4.0, -3.0, -1.0, 2.0, 5.0, 1.25, 0,  8, 12};
    static float b[9] = { 3.0, -3.0,  0.0, 3.0, 4.5, 4.0,  9,  1,  1};
    rtst_log("started length_9_test\n");
    one_test(rtn, 9, a, b, 74.5f);
    rtst_log("finished length_9_test\n");
}

void
run_directed_tests(rtn_t rtn)
{
    length_1_test(rtn);
    length_2_test(rtn);
    length_9_test(rtn);
}

void
run_random_tests(rtn_t rtn)
{
    const int numtests = rtst_numtests;
    const size_t maxlen = rtst_sizetests;
    float *a = malloc_vf(maxlen);
    float *b = malloc_vf(maxlen);

    int i;
    for (i=0; i<numtests; i++)
    {
        float res;
        size_t len = (size_t) rtst_urand_i32i32_i32(1, maxlen);

        rtst_urandi_i32i32_vf(len, -7, +7, a);
        rtst_urandi_i32i32_vf(len, -7, +7, b);
        res = nv_mulsum_vfvf_f(len, a, b);

        rtst_log("len=%d\n", len);

        one_test(rtn, len, a, b, res);
    }

    free_vf(a);
    free_vf(b);
}

int
main(int argc, char *argv[])
{
    rtst_args(argc, argv);

    rtst_start("mulsum_vfvf_f");
    run_directed_tests(nv_mulsum_vfvf_f);
#ifdef CHECK_WRP
    rtst_log("testing wrp.\n");
    run_directed_tests(php_mulsum_vfvf_f);
    run_random_tests(php_mulsum_vfvf_f);
#endif
#ifdef CHECK_PHP
    rtst_log("testing php.\n");
    run_directed_tests(php_mulsum_vfvf_f);
    run_random_tests(php_mulsum_vfvf_f);
#endif
    rtst_passed();
    
    rtst_exit();

    return 0;
}
