/*
   $Header: /u/spert/src/fltvec/testsuite/RCS/sum_vf_f_test.c,v 1.2 1995/07/05 23:47:25 davidj Exp $

   Test floating point vector sum
*/

#include <stddef.h>
#include <stdlib.h>

#include "fltvec_test.h"


typedef float (*rtn_t)(size_t len, const float* in);

/* Performs one test given input values and expected output. */

void
onetest(rtn_t rtn, size_t len,
	const float* in, float correct)
{
    /* Create padded copies of inputs. */
    float* c_in = rtst_padvec_clone_vf(len, in);
    float res;

    /* Run routine. */
    res = rtn(len, c_in);

    /* Check result. */
    rtst_checknear_fvfvf(1, 1e-3, &res, &correct);

    /* Check input not corrupted. */
    rtst_checkeq_vfvf(len, in, c_in);

    /* Free and check padding. */
    rtst_padvec_del_vf(c_in);
}

void
sum_0(rtn_t rtn)
{
    const float* in = NULL;

    onetest(rtn, 0, in, 0.0f);
}

void
sum_1(rtn_t rtn)
{
    const float in[] = { 42.0f };

    onetest(rtn, 1, in, 42.0f);
}

void
sum_3(rtn_t rtn)
{
    const float in[] = { -1.0f, -2.0f, 3.0f };

    onetest(rtn, 3, in, 0.0f);
}

void
sum_32(rtn_t rtn)
{
    const float in[] =
    {
	1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f,
	9.0f, 10.0f, 11.0f, 12.0f, 13.0f, 14.0f, 15.0f, 16.0f,
	17.0f, 18.0f, 19.0f, 20.0f, 21.0f, 22.0f, 23.0f, 24.0f,
	25.0f, 26.0f, 27.0f, 28.0f, 29.0f, 30.0f, 31.0f, 32.0f
    };

    onetest(rtn, 32, in, 33.0f*16.0f);
}

/* Compare the results of two vector routines */

void
random_test(rtn_t nv_rtn, rtn_t test_rtn)
{
    const int numtests = rtst_numtests;
    const size_t maxlen = rtst_sizetests;
    float *in = malloc_vf(maxlen);
    float res;
    
    int i;
    for (i=0; i<numtests; i++)
    {
        size_t len = (size_t) rtst_urand_i32i32_i32(1, maxlen);

        rtst_urand_ff_vf(len, -1e2, 1e2, in);
        res = nv_rtn(len, in);

        rtst_log("len=%d\n", len);

        onetest(test_rtn, len, in, res);
    }

    free_vf(in);
}

void
directed_test(rtn_t rtn)
{
/*     sum_0(rtn); */
    sum_1(rtn);
    sum_3(rtn);
    sum_32(rtn);
}

int
main(int argc, char *argv[])
{
    rtst_args(argc, argv);

    rtst_start("sum_vf_f");
    rtst_log("testing nv.\n");
    directed_test(nv_sum_vf_f);

#ifdef CHECK_TOR
    rtst_log("testing tor.\n");
    directed_test(tor_sum_vf_f);
    random_test(nv_sum_vf_f, tor_sum_vf_f);
#endif

    rtst_passed();

    rtst_exit();

    return 0;
}



