/*
** $Header: /u/spert/src/fltvec/tor/RCS/add_gfgf_gf.S,v 1.6 1995/10/13 03:34:03 davidj Exp $
*/

#include "tor/tordef.h"
#include "m_regdef.h"

/**** Note - this routine is only compile once but has two entry points ***/

	.globl tor_add_gfgf_gf
#ifdef DECL_tor_add_gfgf_gf 
void tor_add_gfgf_gf(void); 
#endif
	.globl tor_sub_gfgf_gf
#ifdef DECL_tor_sub_gfgf_gf 
void tor_sub_gfgf_gf(void); 
#endif

#define vA va0			/* Input A				*/
#define vB va1			/* Input B				*/
#define vZ vv2			/* Output Z				*/

#define vAExp vt2		/* Biased exponent of A			*/
#define vBExp vt3		/* Biased exponent of B			*/
#define vSplitA vt4		/* A with exponent shifted up one bit	*/
#define vSplitB vt5		/* B with exponent shifted up one bit	*/
#define vAdjustedMaxExp vtt	/* max( vAExp, vBExp ) + 2		*/
#define vAlignedASignif vt4	/* Significand of A, aligned for adding	*/
#define vAlignedBSignif vt5	/* Significand of B, aligned for adding	*/
#define vZSign vt0		/* Sign of result			*/
#define vRawZSignif vt2		/* Raw significand of result		*/
#define vCarry vt3		/* Carry from ones-complement addition	*/
#define vRawZSignif2 vt3	/* Raw significand of result		*/
#define vExpAdjust vt4		/* Part of normalization adjustment to	*/
				/*     result exponent			*/
#define vExpAdjust2 vt5		/* More exponent adjustment		*/
#define vLowIndex vv2		/* Index into low zeros-count table	*/
#define vHighIndex vv3		/* Index into high zeros-count table	*/
#define vZSignif vv2		/* Rounded result significand		*/
#define vZExp vt1		/* Under-biased exponent of result	*/
#define vAbsA vt4		/* A shifted up one bit to remove sign	*/
#define vAbsB vt5		/* B shifted up one bit to remove sign	*/
#define vInputsCode vtt		/* Coding of NaN/infinity cases in	*/
				/*     inputs				*/
#define vACode vt4		/* Coding of NaN/infinity cases for A	*/
#define vBCode vt5		/* Coding of NaN/infinity cases for B	*/

#define temp0 v0
#define temp1 v1
#define temp2 t4
#define temp3 t5
#define config0 t6
#define config1 t7
#define config2 t8
#define config3 t9

	.ent tor_add_gfgf_gf

/*
-------------------------------------------------------------------------------
Routine:  tor_add_gfgf_gf

Input:
	vlr
	va0	vA
	va1	vB

Output:
	vv2	vZ

Not preserved:
	vtt
	vv3
	va0--va3
	vt0--vt5
	v0--v1
	t4--t9

Function:

The single-precision floating-point vectors vA and vB are element-wise added
and the single-precision result vector returned in vZ.  The vlr register
controls the elements of vZ actually updated.  This operation conforms to the
IEEE Standard for Binary Floating-point Arithmetic, except as follows:
(1) Only nearest/even rounding is supported; (2) No inexact flags are raised;
and (3) No signalling NaNs are recognized.  All NaNs are quiet.  For all cases
in which the result of the subtraction is a NaN, the default MIPS NaN is
returned.  The bit patterns of input NaNs are not preserved.

Exceptions for any of the element-wise additions are signalled by raising
element-wise flags in the global variables fltflags_invalid32 and
fltflags_overflow32.  No traps are taken for any exceptions.

-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
Routine:  tor_sub_gfgf_gf

Input:
	vlr
	va0	vA
	va1	vB

Output:
	vv2	vZ

Not preserved:
	vtt
	vv3
	va0--va3
	vt0--vt5
	v0--v1
	t4--t9

Function:

The single-precision floating-point vectors vA and vB are element-wise
subtracted and the single-precision result vector returned in vZ.  The vlr
register controls the elements of vZ actually updated.  This operation conforms
to the IEEE Standard for Binary Floating-point Arithmetic, except as follows:
(1) Only nearest/even rounding is supported; (2) No inexact flags are raised;
and (3) No signalling NaNs are recognized.  All NaNs are quiet.  For all cases
in which the result of the subtraction is a NaN, the default MIPS NaN is
returned.  The bit patterns of input NaNs are not preserved.

Exceptions for any of the element-wise subtractions are signalled by
raising element-wise flags in the global variables fltflags_invalid32 and
fltflags_overflow32.  No traps are taken for any exceptions.

-------------------------------------------------------------------------------
*/

tor_sub_gfgf_gf:

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
|	va0	vA
|	va1	vB
|	vt2	vAExp
|	vt3	vBExp
|	vt4	vSplitA
|	vt5	vSplitB
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Invert the sign of B and fall directly into the addition routine.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	li temp0,0x80000000
	xor.vs vB,vB,temp0

tor_add_gfgf_gf:

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
|	va0	vA
|	va1	vB
|	vt2	vAExp
|	vt3	vBExp
|	vt4	vSplitA
|	vt5	vSplitB
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Form vSplitA by shifting the exponent of A up by one bit in the word.  This
eliminates the sign and makes room for an explicit leading significand bit.
Make the leading significand bit explicit by setting the bit between the
exponent and the rest of the significand if A is not subnormal.  Do all the
same for B.
    Infinity and NaN inputs are (mis)treated as though they were normal.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	li config0,(FX_A_AND_B|FX_CLIP_PASS)
	li temp0,0xFF800000
	fxadd.vs vSplitA,vA,temp0,config0
	fxadd.vs vSplitB,vB,temp0,config0
	li config1,(FX_SHLZA|(25*FX_SHLAM)|FX_A_OR_B|FX_BU\
	            |FX_SEPSHAM|FX_LURES|FX_CLIP_PASS|FX_LTZ)
	li temp1,(1<<23)
	fxsub.vs vSplitA,vSplitA,temp1,config1
	fxsub.vs vSplitB,vSplitB,temp1,config1

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Extract the biased exponent of A, or set the biased exponent to 1 if A is
subnormal.  (The existence of a leading 1 significand bit tells whether A is
subnormal.)  Do the same for B.
    Infinity and NaN inputs continue to be (mis)treated as though they were
normal.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	li config2,(FX_A_XOR_B|FX_BU\
	            |FX_SEPSHAM|(24*FX_SHRAM)|FX_NOROUND|FX_CLIP_PASS)
	fxadd.sv vAExp,temp1,vSplitA,config2
	fxadd.sv vBExp,temp1,vSplitB,config2

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Set up a "compare, conditional move" pipeline configuration, used multiple
times.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	li config0,(FX_AU|FX_BU|FX_LURES|FX_CLIP_PASS|FX_LEZ)

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
|	vtt	vAdjustedMaxExp
|	va0	vA
|	va1	vB
|	vt2	vAExp
|	vt3	vBExp
|	vt4	vSplitA, vAlignedASignif
|	vt5	vSplitB, vAlignedBSignif
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Set vAdjustedMaxExp to the maximum of vAExp and vBExp, plus 2.  (vAExp is
recalculated to improve instruction scheduling.  The maximum exponent is the
tentative result exponent.  The adjustment by 2 is critical for proper
alignment of the significands below.)  If one of the inputs is an infinity or
a NaN, vAdjustedMaxExp will be 0xFF + 2 = 0x101.  (This is used below to test
for such inputs.)
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	fxadd.sv vAdjustedMaxExp,temp1,vSplitA,config2
	fxsub.vv vAdjustedMaxExp,vAdjustedMaxExp,vBExp,config0
	li temp0,2
	addu.vs vAdjustedMaxExp,temp0

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Extract and align the significands as follows:  Calculate how many bits to
shift down the significand of A by subtracting vAExp from vAdjustedMaxExp.
(vAdjustedMaxExp >= vAExp + 2, so this value will be at least 2.)  To ensure a
proper shift for large shift amounts, saturate the shift amount to a maximum
of 31.  Extract the significand of A by shifting up, and then shift down by the
calculated amount, rounding to odd (jamming).  Do all the same for B.
    (There must be at least 2 free bits at the top of vAlignedASignif and
vAlignedBSignif to allow for meaningful signs and to provide "headroom" for the
addition.  There must also be some room at the bottom to ensure correct
rounding of the result.  Only one of the significands will actually be shifted
down more than 2 bits.)
    Infinity and NaN inputs continue to be (mis)treated as though they were
normal.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	subu.vv vAExp,vAdjustedMaxExp,vAExp
	subu.vv vBExp,vAdjustedMaxExp,vBExp
	li config1,(FX_LURES|FX_CLIP_PASS|FX_GTZ)
	li temp1,31
	fxsub.vs vAExp,vAExp,temp1,config1
	fxsub.vs vBExp,vBExp,temp1,config1
	li config2,((8*FX_SHLAM)|FX_AU|FX_SHRL|FX_SHRV|FX_RODD|FX_CLIP_PASS)
	fxadd.vv vAlignedASignif,vSplitA,vAExp,config2
	fxadd.vv vAlignedBSignif,vSplitB,vBExp,config2

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
|	vtt	vAdjustedMaxExp
|	va0	vA
|	va1	vB
|	vt0	vZSign
|	vt2	vRawZSignif
|	vt3	vCarry
|	vt4	vAlignedASignif
|	vt5	vAlignedBSignif
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Complement either or both aligned significands depending on the signs of the
original inputs.  Add the aligned significands using ones-complement addition.
Note the sign of the result, and take the absolute value of the significand sum
to return to sign-magnitude representation.
    (To ensure that zeros of either sign are handled properly, the addition is
performed using _ones-complement_ arithmetic, which has separate
representations for positive and negative zeros.  Ordinarily, negative values
are encoded in ones-complement by complementing all the bits of the
corresponding positive value; however, in the following code the interpretation
of the ones-complement signs is reversed from the usual convention, in order
that the subtraction of equal magnitudes gives a result that will be
interpreted as positive zero.  _Positive_ values are complemented, and negative
values are not.
    (For example--assuming integers--the arithmetic works as follows:

	+3 + -5 = 0xFFFFFFFC + 0x00000005 = 0x00000002 = -2;
	+3 + -3 = 0xFFFFFFFC + 0x00000003 = 0xFFFFFFFF = +0.

Significantly,

	+0 + +0 = 0xFFFFFFFF + 0xFFFFFFFF = 0xFFFFFFFF = +0;
	+0 + -0 = 0xFFFFFFFF + 0x00000000 = 0xFFFFFFFF = +0;
	-0 + +0 = 0x00000000 + 0xFFFFFFFF = 0xFFFFFFFF = +0;
	-0 + -0 = 0x00000000 + 0x00000000 = 0x00000000 = -0.

    (Note from the examples that ones-complement addition is not identical to
twos-complement addition.  Performing a ones-complement addition entails first
adding the two operands as unsigned values to form a tentative sum.  If there
was no carry out from this unsigned addition, the tentative sum is exactly the
ones-complement sum; otherwise, the tentative sum must be incremented to form
the ones-complement sum.  This can be written as

	carry,sum := a + b;		--Unsigned addition with carry.
	sum := sum + carry;		--Wrap around any carry.

in pseudo-code.)
    Infinity and NaN inputs continue to be (mis)treated as though they were
normal.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	li config3,(FX_NOT_B|FX_SHRV|FX_LURES|FX_CLIP_PASS|FX_GEZ)
	fxadd.vv vAlignedASignif,vA,vAlignedASignif,config3
	fxadd.vv vAlignedBSignif,vB,vAlignedBSignif,config3
	addu.vv vRawZSignif,vAlignedASignif,vAlignedBSignif
	li config2,(FX_AU|FX_BU|FX_CLIP_SLTZ)
	fxadd.vv vCarry,vAlignedASignif,vAlignedBSignif,config2
	not.vv vZSign,vRawZSignif
	addu.vv vRawZSignif,vRawZSignif,vCarry
	fxadd.vv vRawZSignif,vZSign,vRawZSignif,config3
	sltz.vv vZSign,vZSign

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
|	vtt	vAdjustedMaxExp
|	vv2	vLowIndex, vZSignif
|	vv3	vHighIndex
|	va0	vA
|	va1	vB
|	vt0	vZSign
|	vt2	vRawZSignif
|	vt3	vRawZSignif2
|	vt4	vExpAdjust
|	vt5	vExpAdjust2
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Set vExpAdjust to the number of leading zeros of vRawZSignif, and shift up to
normalize.
    (NOTE:  This block of code could all be replaced by a count-leading-zeros
instruction followed by a shift.  See countlz_gi32_gi32.S for more refined
commentary.)
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	li temp2,(1<<16)
	li temp3,(1<<24)
	li config1,(FX_SHLZA|FX_A_AND_B|(23*FX_SHLAM)|FX_NOROUND|FX_CLIP_PASS)
	li config2,(FX_SHLV|(28*FX_SHLAM)|FX_NOROUND|FX_CLIP_PASS|FX_GEZ)
	sltu.vs vExpAdjust2,vRawZSignif,temp2
	li temp0,16
	sllv.vs vRawZSignif2,vRawZSignif,temp0
	fxsub.sv vRawZSignif2,temp2,vRawZSignif,config0
	sltu.vs vExpAdjust,vRawZSignif2,temp3
	li config3,((1*FX_SHLAM)|FX_SEPSHAM|FX_NOROUND)
	fxadd.vv vExpAdjust2,vExpAdjust2,vExpAdjust,config3
	li temp1,8
	sllv.vs vRawZSignif,vRawZSignif2,temp1
	fxsub.sv vRawZSignif,temp3,vRawZSignif2,config0
	li temp2,(7<<25)
	fxadd.vs vLowIndex,vRawZSignif,temp2,config1
	li temp3,0x76554444
	fxadd.sv vExpAdjust,temp3,vLowIndex,config2
	addu config1,(3*FX_SHLAM)
	li temp0,(7<<28)
	fxadd.vs vHighIndex,vRawZSignif,temp0,config1
	li temp1,0xF3221111
	fxadd.sv vExpAdjust,temp1,vHighIndex,config2
	cmvltz.vv vExpAdjust,vRawZSignif,vzero
	sllv.vv vRawZSignif,vExpAdjust
	li config0,((3*FX_SHLAM)|FX_SEPSHAM|FX_NOROUND)
	fxadd.vv vExpAdjust,vExpAdjust2,vExpAdjust,config0

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Form the tentative result significand by shifting the raw result significand
down, rounding to nearest/even.
    Infinity and NaN inputs continue to be (mis)treated as though they were
normal.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	li config1,(FX_BU|FX_SHRL|(8*FX_SHLAM)|FX_RNEARESTEVEN|FX_CLIP_PASS)
	fxadd.vv vZSignif,vzero,vRawZSignif,config1

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
|	vtt	vAdjustedMaxExp
|	vv2	vZSignif
|	va0	vA
|	va1	vB
|	vt0	vZSign
|	vt1	vZExp
|	vt2	vRawZSignif
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Calculate the "under-biased" exponent of the result as vAdjustedMaxExp -
vExpAdjust - 1.  (The exponent is "under-biased" because the integral portion
of the significand will be added to it when the result is packed.  In most
cases, a 1 will be added to the exponent during packing.  However, if a carry
occurred during rounding of the significand, a value of 2 will be added to the
exponent during packing.)  If the result significand is exactly zero, replace
the result exponent by zero.  (If the significand is zero, the value added to
the exponent during packing will be zero.)
    Infinity and NaN inputs continue to be (mis)treated as though they were
normal.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	li config2,(FX_NOT_B|FX_CLIP_PASS)
	fxadd.vv vZExp,vAdjustedMaxExp,vExpAdjust,config2
	cmveqz.vv vZExp,vZSignif,vzero

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
If vAdjustedMaxExp equals 0xFF + 2, one or both of the original inputs was a
NaN or infinity.  If vZExp is less than 0, the result is a subnormal or has
underflowed.  If vZExp is 0xFD or more, the result might have overflowed.  If
any of these cases holds for any vector element, jump to a special case
handler.
    (Although the result has not actually overflowed in most cases in which
vZExp equals 0xFD, overflow does occur when vZExp is 0xFD and vRawZSignif >=
0xFFFFFF80.)
    (Since a zero significand forced a zero exponent earlier, an exact zero
result will not cause a jump to the special case handler.  In all other cases,
if vZExp is zero, the biased exponent of the result will be at least 1.)
    (Note that temp2 is still live at the entrance to the special cases code.)
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	ctvu zero,vcond
	li temp2,(0xFF+2)
	feq.vs vAdjustedMaxExp,temp2
	cfvu temp2,vcond
	li temp3,0xFC
	fltu.sv temp3,vZExp
	cfvu temp3,vcond
	or temp3,temp2
	bnez temp3,specialCases

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
|	vv2	vZSignif, vZ
|	vt0	vZSign
|	vt1	vZExp
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Add the integral portion of the significand to the under-biased exponent, and
pack the sign, biased exponent, and significand of the result.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	li config0,((23*FX_SHLAM)|FX_SEPSHAM|FX_NOROUND)
	fxadd.vv vZ,vZExp,vZSignif,config0
	li config1,((31*FX_SHLAM)|FX_SEPSHAM|FX_NOROUND)
	fxadd.vv vZ,vZSign,vZ,config1

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Return.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	j ra

specialCases:

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
|	vv2	vZSignif
|	va0	vA
|	va1	vB
|	vt0	vZSign
|	vt1	vZExp
|	vt2	vRawZSignif
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
If both inputs are finite, the only possible special cases involve overflow or
underflow of the result.  If this is the case for every vector element, skip
the code to handle NaN and infinity inputs.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	beqz temp2,resultSpecialCasesOnly

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
|	vtt	vInputsCode
|	vv2	vZSignif
|	va0	vA
|	va1	vB
|	vt0	vZSign
|	vt1	vZExp
|	vt2	vRawZSignif
|	vt4	vAbsA,vACode
|	vt5	vAbsB,vBCode
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Distill the types of the original inputs into a single value in vInputsCode.
Set vInputsCode to between 3 and 6 inclusive if either input is a NaN; else set
vInputsCode to 0 or 1 if either input is an infinity; else set vInputsCode to
exactly 2.  (This distillation is done by first setting vACode to either 1, 0,
or 3, according to whether the original A input is normal, an infinity, or a
NaN, respectively.  The same is done with vBCode, and then vACode and vBCode
are added to form vInputsCode.  This value is used by the special case handler
called below.)
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	addu.vv vAbsA,vA,vA
	addu.vv vAbsB,vB,vB
	li config0,(FX_AU|FX_BU|FX_SHRL|FX_SEPSHAM|(31*FX_SHRAM)|FX_RODD)
	li temp0,(0xFF<<24)
	fxsub.sv vACode,temp0,vAbsA,config0
	fxsub.sv vBCode,temp0,vAbsB,config0
	addu.vv vInputsCode,vACode,vBCode

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
|	vtt	vInputsCode
|	vv2	vZSignif
|	vt0	vZSign
|	vt1	vZExp
|	vt2	vRawZSignif
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Continue with the special case input and result handler common to both single-
precision floating-point addition and multiplication.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	j tor_fltv_singlespec

resultSpecialCasesOnly:

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
|	vv2	vZSignif
|	vt0	vZSign
|	vt1	vZExp
|	vt2	vRawZSignif
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Continue with the special case result handler common to all single-precision
floating-point operations.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	j tor_fltv_singlerange

	.end tor_add_gfgf_gf

